var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
import { BehaviorSubject as BehaviorSubject$1 } from 'rxjs/BehaviorSubject';
import { AsyncSubject as AsyncSubject$1 } from 'rxjs/AsyncSubject';
import { Inject, InjectionToken, NgModule, Pipe } from '@angular/core';
import { CommonModule } from '@angular/common';
import { createStore } from 'redux';
var METADATA_KEY$1 = Symbol('@ReduxActionController');
var METADATA_DEFAULT$1 = {
    actionPrefix: '',
};
/**
 * @param {?} actionPrefix
 * @return {?}
 */
function ReduxActionController(actionPrefix) {
    return function (constructor) {
        var /** @type {?} */ metadata = { actionPrefix: actionPrefix };
        Reflect['defineMetadata'](METADATA_KEY$1, metadata, constructor);
        return constructor;
    };
}
/**
 * @param {?} target
 * @return {?}
 */
function getReduxActionControllerMetadata(target) {
    return Object.assign({}, METADATA_DEFAULT$1, Reflect['getMetadata'](METADATA_KEY$1, target));
}
/**
 * @param {?} action
 * @return {?}
 */
function dispatch(action) {
    ReduxRegistry.getStore().then(function (store) { return store.dispatch(action); });
}
var METADATA_KEY = Symbol('@ReduxActionController');
var METADATA_DEFAULT = {
    methodType: '',
    targetConstructor: null,
};
/**
 * @param {?=} customType
 * @return {?}
 */
function ReduxAction(customType) {
    return function (target, propertyKey, descriptor) {
        var /** @type {?} */ methodType = customType || propertyKey;
        var /** @type {?} */ method = descriptor.value;
        var /** @type {?} */ proxyFunction = function () {
            var /** @type {?} */ metadata = getReduxActionControllerMetadata(target.constructor);
            var /** @type {?} */ type = getActionType(metadata.actionPrefix, methodType);
            var /** @type {?} */ payload = method.apply(this, arguments);
            if (payload instanceof Promise) {
                payload.then(function (p) { return dispatch({ payload: p, type: type }); });
            }
            else {
                dispatch({ payload: payload, type: type });
            }
            return payload;
        };
        Reflect['defineMetadata'](METADATA_KEY, {
            methodType: methodType,
            targetConstructor: target.constructor,
        }, proxyFunction);
        descriptor.value = proxyFunction;
    };
}
/**
 * @param {?} target
 * @return {?}
 */
function getReduxActionMetadata(target) {
    return Object.assign({}, METADATA_DEFAULT, Reflect['getMetadata'](METADATA_KEY, target));
}
/**
 * @param {?} type
 * @return {?}
 */
function getActionTypeByFunction(type) {
    var _a = getReduxActionMetadata(type), methodType = _a.methodType, targetConstructor = _a.targetConstructor;
    var actionPrefix = getReduxActionControllerMetadata(targetConstructor).actionPrefix;
    return getActionType(actionPrefix, methodType);
}
/**
 * @param {...?} rest
 * @return {?}
 */
function getActionType() {
    var rest = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        rest[_i] = arguments[_i];
    }
    return rest.filter(function (i) { return i.length > 0; }).join('/');
}
var ReduxRegistryReducerItem = /** @class */ (function () {
    function ReduxRegistryReducerItem() {
    }
    return ReduxRegistryReducerItem;
}());
var ReduxRegistry = /** @class */ (function () {
    function ReduxRegistry() {
    }
    /**
     * @param {?} store
     * @return {?}
     */
    ReduxRegistry.registerStore = function (store) {
        ReduxRegistry._store.next(store);
        ReduxRegistry._store.complete();
    };
    /**
     * @param {?} stateName
     * @param {?} types
     * @param {?} reducer
     * @return {?}
     */
    ReduxRegistry.registerReducer = function (stateName, types, reducer) {
        if (Array.isArray(types)) {
            types.forEach(function (type) {
                ReduxRegistry._reducers.push({
                    reducer: reducer,
                    stateName: stateName,
                    type: type,
                });
            });
        }
        else {
            ReduxRegistry._reducers.push({
                reducer: reducer,
                stateName: stateName,
                type: /** @type {?} */ (types),
            });
        }
    };
    /**
     * @param {?} stateName
     * @param {?} module
     * @return {?}
     */
    ReduxRegistry.registerModule = function (stateName, module) {
        ReduxRegistry.getStore().then(function (store) {
            var /** @type {?} */ initialState = {};
            if (module.onReduxInit) {
                initialState = module.onReduxInit() || {};
            }
            store.dispatch({
                payload: {
                    initialState: initialState,
                    stateName: stateName,
                },
                type: ReduxRegistry.ACTION_REGISTER_MODULE,
            });
        });
    };
    /**
     * @param {?} type
     * @return {?}
     */
    ReduxRegistry.getReducerItemsByType = function (type) {
        return ReduxRegistry._reducers
            .filter(function (reducerItem) { return getActionTypeByFunction(reducerItem.type) === type; });
    };
    /**
     * @return {?}
     */
    ReduxRegistry.getStore = function () {
        return new Promise(ReduxRegistry._store.subscribe.bind(ReduxRegistry._store));
    };
    return ReduxRegistry;
}());
ReduxRegistry.ACTION_REGISTER_MODULE = ReduxRegistry.name + "://registerModule";
ReduxRegistry._store = new AsyncSubject$1();
ReduxRegistry._reducers = [];
/**
 * @template S
 * @param {?} state
 * @param {?} selector
 * @return {?}
 */
function selectByState(state, selector) {
    var /** @type {?} */ selectorArray = selector.split('/');
    if (selectorArray[0] !== '') {
        throw new Error('Relative selectors are not supported yet!');
    }
    return selectorArray
        .filter(function (propertyKey) { return propertyKey !== ''; })
        .reduce(function (previousValue, propertyKey) {
        if (!previousValue || !previousValue[propertyKey]) {
            return null;
        }
        return previousValue[propertyKey];
    }, state);
}
/**
 * @template S
 * @param {?} selector
 * @return {?}
 */
function select(selector) {
    var /** @type {?} */ subject = new BehaviorSubject$1(null);
    ReduxRegistry.getStore().then(function (store) {
        subject.next(selectByState(store.getState(), selector));
        store.subscribe(function () { return subject.next(selectByState(store.getState(), selector)); });
    });
    return subject.asObservable();
}
var METADATA_KEY$2 = Symbol('@ReduxActionController');
var METADATA_DEFAULT$2 = {
    reducers: [],
};
/**
 * @template S, P
 * @param {?} types
 * @return {?}
 */
function ReduxReducer(types) {
    return function (target, propertyKey) {
        var /** @type {?} */ metadata = getReduxReducerClassMetadata(target.constructor);
        metadata.reducers.push({ types: types, reducer: target[propertyKey].bind(target) });
        Reflect['defineMetadata'](METADATA_KEY$2, metadata, target.constructor);
        if (typeof target === 'object') {
            target[propertyKey].bind(target);
        }
        if (!target['__@ReduxReducer']) {
            target['__@ReduxReducer'] = [];
        }
        return target[propertyKey];
    };
}
/**
 * @param {?} target
 * @return {?}
 */
function getReduxReducerClassMetadata(target) {
    return Object.assign({}, METADATA_DEFAULT$2, Reflect['getMetadata'](METADATA_KEY$2, target));
}
/**
 * @param {?=} config
 * @return {?}
 */
function Redux(config) {
    return function (constructor) {
        return /** @class */ (function (_super) {
            __extends(class_1, _super);
            function class_1() {
                var _this = _super.apply(this, arguments) || this;
                _this.redux = (function () {
                    var /** @type {?} */ mergedConfig = Object.assign({
                        reducers: [],
                        stateName: '',
                    }, config || {});
                    var stateName = mergedConfig.stateName, reducers = mergedConfig.reducers;
                    // constructor[ '__@Redux' ] = mergedConfig;
                    ReduxRegistry.registerModule(stateName, _this);
                    reducers
                        .map(function (reducer) { return getReduxReducerClassMetadata(reducer.constructor); })
                        .forEach(function (metadata) {
                        metadata.reducers.forEach(function (reducer) {
                            ReduxRegistry.registerReducer(stateName, reducer.types, reducer.reducer);
                        });
                    });
                    return config;
                })();
                return _this;
            }
            return class_1;
        }(constructor));
    };
}
/**
 * @param {?} selector
 * @return {?}
 */
function ReduxSelect(selector) {
    return function (target, propertyKey) {
        Object.defineProperty(target, propertyKey, {
            configurable: true,
            enumerable: true,
            value: select(selector),
        });
    };
}
var ReduxSelectPipe = /** @class */ (function () {
    function ReduxSelectPipe() {
    }
    /**
     * @param {?} selector
     * @return {?}
     */
    ReduxSelectPipe.prototype.transform = function (selector) {
        return select(selector);
    };
    return ReduxSelectPipe;
}());
ReduxSelectPipe.decorators = [
    { type: Pipe, args: [{
                name: 'reduxSelect',
            },] },
];
/**
 * @nocollapse
 */
ReduxSelectPipe.ctorParameters = function () { return []; };
var rootReducer = function (state, action) {
    if (action.type === ReduxRegistry.ACTION_REGISTER_MODULE) {
        var _a = action.payload, stateName = _a.stateName, initialState = _a.initialState;
        return Object.assign({}, state, (_b = {},
            _b[stateName] = initialState,
            _b));
    }
    return ReduxRegistry.getReducerItemsByType(action.type)
        .reduce(function (stateToReduce, reducerItem) {
        var stateName = reducerItem.stateName, reducer = reducerItem.reducer;
        return Object.assign({}, stateToReduce, (_a = {},
            _a[stateName] = reducer(stateToReduce[stateName], action),
            _a));
        var _a;
    }, state);
    var _b;
};
var REDUX_MODULE_CONFIG = new InjectionToken('REDUX_MODULE_CONFIG');
var IS_ROOT_MODULE = new InjectionToken('IS_ROOT_MODULE');
var ReduxModule = /** @class */ (function () {
    /**
     * @param {?=} isRootModule
     * @param {?=} config
     */
    function ReduxModule(isRootModule, config) {
        if (isRootModule === void 0) { isRootModule = false; }
        if (config === void 0) { config = null; }
        if (isRootModule) {
            config = Object.assign({
                store: null,
            }, config || {});
            ReduxRegistry.registerStore(config.store || createStore(rootReducer, {}));
        }
    }
    /**
     * @return {?}
     */
    ReduxModule.forChild = function () {
        return {
            ngModule: ReduxModule,
            providers: [
                { provide: IS_ROOT_MODULE, useValue: false },
            ],
        };
    };
    /**
     * @param {?=} config
     * @return {?}
     */
    ReduxModule.forRoot = function (config) {
        return {
            ngModule: ReduxModule,
            providers: [
                { provide: REDUX_MODULE_CONFIG, useValue: config },
                { provide: IS_ROOT_MODULE, useValue: true },
            ],
        };
    };
    return ReduxModule;
}());
ReduxModule.decorators = [
    { type: NgModule, args: [{
                declarations: [
                    ReduxSelectPipe,
                ],
                exports: [
                    ReduxSelectPipe,
                ],
                imports: [
                    CommonModule,
                ],
                providers: [],
            },] },
];
/**
 * @nocollapse
 */
ReduxModule.ctorParameters = function () { return [
    { type: undefined, decorators: [{ type: Inject, args: [IS_ROOT_MODULE,] },] },
    { type: undefined, decorators: [{ type: Inject, args: [REDUX_MODULE_CONFIG,] },] },
]; };
/**
 * Generated bundle index. Do not edit.
 */
export { selectByState, select, ReduxAction, getReduxActionMetadata, getActionTypeByFunction, getActionType, ReduxActionController, getReduxActionControllerMetadata, ReduxReducer, getReduxReducerClassMetadata, Redux, ReduxSelect, ReduxSelectPipe, REDUX_MODULE_CONFIG, IS_ROOT_MODULE, ReduxModule, ReduxRegistryReducerItem, ReduxRegistry, rootReducer };
//# sourceMappingURL=ngx-redux-core.es5.js.map
